/**
 * Copyright (c) 2011, Regents of the University of California
 * All rights reserved.
 * <p/>
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * <p/>
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * <p/>
 * 2. Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * <p/>
 * 3. Neither the name of the University of California, Berkeley nor
 * the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * <p/>
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package edu.berkeley.cs.wise.concolic;

import java.io.Serializable;

/**
 * @author Jacob Burnim <jburnim@cs.berkeley.edu>
 * @author Koushik Sen <ksen@cs.berkeley.edu>
 */
public class Branch implements Comparable<Branch>, Serializable {

    public Branch(int iid, boolean branch) {
        this.iid = iid;
        this.branch = branch;
    }

    public int getIid() {
        return iid;
    }

    public boolean getBranch() {
        return branch;
    }

    public Branch getFlippedBranch() {
        return new Branch(iid, !branch);
    }

    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + (branch ? 1231 : 1237);
        result = prime * result + iid;
        return result;
    }

    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null)
            return false;
        if (getClass() != o.getClass())
            return false;
        Branch other = (Branch)o;
        if (branch != other.branch)
            return false;
        if (iid != other.iid)
            return false;
        return true;
    }

    public int compareTo(Branch b) {
        if (iid < b.iid)
            return -1;
        else if (iid > b.iid)
            return 1;
        else if (!branch && b.branch)
            return -1;
        else if (branch && !b.branch)
            return 1;
        else
            return 0;
    }

    public String toString() {
        return iid + (branch ? "(T)" : "(F)");
    }

    private final int iid;
    private final boolean branch;
}
