/**
 * Copyright (c) 2011, Regents of the University of California
 * All rights reserved.
 * <p/>
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * <p/>
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * <p/>
 * 2. Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * <p/>
 * 3. Neither the name of the University of California, Berkeley nor
 * the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * <p/>
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package edu.berkeley.cs.wise.concolic;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

/**
 * @author Koushik Sen <ksen@cs.berkeley.edu>
 */
public class SymbolicState {
    private Map<Long, Expression> globals;
    private Map<Long, Expression> top;
    private LinkedList<Map<Long, Expression>> locals;


    public SymbolicState() {
        globals = new HashMap<Long, Expression>();
        locals = new LinkedList<Map<Long, Expression>>();
        top = new HashMap<Long, Expression>();
        locals.addFirst(top);
    }

    public Expression getExpression(long l) {
        if (l == 0) {
            return null;
        } else if ((l >> 32) == 0) { // local variable
            return top.get(l);
        } else {
            return globals.get(l);
        }
    }

    public void setExpression(long l, Expression e) {
        if (l == 0) {

        } else if ((l >> 32) == 0) { // local variable
            if (e == null) {
                top.remove(l);
            } else {
                top.put(l, e);
            }
        } else {
            if (e == null) {
                globals.remove(l);
            } else {
                globals.put(l, e);
            }
        }
    }

    public void push() {
        top = new HashMap<Long, Expression>();
        locals.addFirst(top);
    }

    public void pop() {
        locals.removeFirst();
        if (locals.isEmpty())
            top = null;
        else
            top = locals.getFirst();
    }

    public void reset() {
        globals.clear();
        locals.clear();
        top = new HashMap<Long, Expression>();
        locals.addFirst(top);
    }
}
