/**
 * Copyright (c) 2011, Regents of the University of California
 * All rights reserved.
 * <p/>
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * <p/>
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * <p/>
 * 2. Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * <p/>
 * 3. Neither the name of the University of California, Berkeley nor
 * the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * <p/>
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package edu.berkeley.cs.wise.concolic;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @author Koushik Sen <ksen@cs.berkeley.edu>
 */
public class UnitTask implements Runnable, Comparable {
    private History history;
    private String[] args;
    private String cFile;
    private int queueSize;
    private ThreadPoolExecutor pool;
    private TaskCounter tc;


    public UnitTask(History history, String[] args, String cFile, int queueSize, ThreadPoolExecutor pool, TaskCounter tc) {
        this.history = history;
        this.args = args;
        this.cFile = cFile;
        this.queueSize = queueSize;
        this.pool = pool;
        this.tc = tc;
    }

    public void run() {
        ObserverForConcolic.eh.set(new ConcolicEventHandler(history));
        //if (history!=null) history.print();
        Method m = null;
        try {
            Class c = Class.forName(cFile);
            Class[] argTypes = {args.getClass(),};
            m = c.getMethod("main", argTypes);

        } catch (Exception e) {
            System.err.println(e);
            System.exit(1);
        }

        Object passedArgv[] = {args};
        int j = 0;
        do {
            j++;
            //System.out.println("******** Iteration " + j + " ********" + Thread.currentThread());
            try {
                m.invoke(null, passedArgv);
            } catch (IllegalAccessException e) {
                System.err.println("Got Exception");
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                System.err.println("Got Exception");
                e.printStackTrace();
            }
            if (tc.getCount() < queueSize) {
                History hist = ObserverForConcolic.CreateTask();
                if (hist != null) {
                    //hist.print();
                    tc.increment();
                    pool.execute(new UnitTask(hist, args, cFile, queueSize, pool, tc));
                }
            }
        } while (ObserverForConcolic.SolveAndReset());
        tc.decrement();
    }


    public int compareTo(Object o) {
        return history.compare(history, ((UnitTask) o).history);
    }
}
