/**
 * Copyright (c) 2011, Regents of the University of California
 * All rights reserved.
 * <p/>
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * <p/>
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * <p/>
 * 2. Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * <p/>
 * 3. Neither the name of the University of California, Berkeley nor
 * the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * <p/>
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package edu.berkeley.cs.wise.concolic.generators;

import edu.berkeley.cs.wise.concolic.History;

/**
 * @author Jacob Burnim <jburnim@cs.berkeley.edu>
 */
public class TrivialGenerator
    implements Generator, BoundedJoinSemilattice<TrivialGenerator> {

    public static class Factory implements GeneratorFactory<TrivialGenerator> {
	public TrivialGenerator makeGenerator(History hist) {
	    return INSTANCE;
	}
    }

    private static final TrivialGenerator INSTANCE = new TrivialGenerator();

    private TrivialGenerator() { }

    public boolean isLessThan(TrivialGenerator g) {
	return false;
    }

    public boolean isLessThanOrEqualTo(TrivialGenerator g) {
	return true;
    }

    public TrivialGenerator join(TrivialGenerator g) {
        return INSTANCE;
    }

    public TrivialGenerator getBottomElement() {
        return INSTANCE;
    }

    // Built in equals and hashCode are correct for the single INSTANCE.

    public int findFirstIllegalBranch(History h) {
	return -1;
    }
}
